@@ -55,7 +55,9 @@ use plottable::Plottable;
 mod render;
 pub mod util;
 
+pub mod bounded_iter;
 pub mod buffered_iter;
+
 pub mod plotnum;
 use plotnum::*;
 pub mod num;
@@ -79,50 +81,55 @@ const WIDTH: f64 = 800.0;
 ///The height of the svg tag.
 const HEIGHT: f64 = 500.0;
 
-trait PlotTrait<X: PlotNum, Y: PlotNum> {
+trait PlotTrait {
+    type Item;
+    fn plot_type(&self) -> PlotType;
     fn write_name(&self, a: &mut dyn fmt::Write) -> fmt::Result;
-    fn iter_first(&mut self) -> &mut dyn Iterator<Item = (X, Y)>;
-    fn iter_second(&mut self) -> &mut dyn Iterator<Item = (X, Y)>;
+    fn iter_first(&mut self) -> &mut dyn Iterator<Item = Self::Item>;
+    fn iter_second(&mut self) -> &mut dyn Iterator<Item = Self::Item>;
 }
 
-struct PlotStruct<X: PlotNum, Y: PlotNum, I: PlotIter<Item = (X, Y)>, F: Display> {
+struct PlotStruct<I: PlotIter, F: Display> {
+    ptype: PlotType,
     iter: Option<I>,
     it1: Option<I::It1>,
     it2: Option<I::It2>,
     func: F,
-    _p: PhantomData<(X, Y)>,
 }
 
-impl<X: PlotNum, Y: PlotNum, I: PlotIter<Item = (X, Y)>, F: Display> PlotStruct<X, Y, I, F> {
-    fn new(iter: I, func: F) -> Self {
+impl<I: PlotIter, F: Display> PlotStruct<I, F> {
+    fn new(iter: I, func: F, ptype: PlotType) -> Self {
         PlotStruct {
             iter: Some(iter),
             it1: None,
             it2: None,
             func,
-            _p: PhantomData,
+            ptype,
         }
     }
 }
 
-impl<X: PlotNum, Y: PlotNum, D: PlotIter<Item = (X, Y)>, F: Display> PlotTrait<X, Y>
-    for PlotStruct<X, Y, D, F>
-{
+impl<X, Y, D: PlotIter<Item1 = (X, Y), Item2 = (X, Y)>, F: Display> PlotTrait for PlotStruct<D, F> {
+    type Item = (X, Y);
+    fn plot_type(&self) -> PlotType {
+        self.ptype
+    }
     fn write_name(&self, a: &mut dyn fmt::Write) -> fmt::Result {
         write!(a, "{}", self.func)
     }
-    fn iter_first(&mut self) -> &mut dyn Iterator<Item = (X, Y)> {
+    fn iter_first(&mut self) -> &mut dyn Iterator<Item = Self::Item> {
         self.it1 = Some(self.iter.as_mut().unwrap().first());
         self.it1.as_mut().unwrap()
     }
 
-    fn iter_second(&mut self) -> &mut dyn Iterator<Item = (X, Y)> {
+    fn iter_second(&mut self) -> &mut dyn Iterator<Item = Self::Item> {
         let j = self.iter.take().unwrap().second(self.it1.take().unwrap());
         self.it2 = Some(j);
         self.it2.as_mut().unwrap()
     }
 }
 
+#[derive(Copy, Clone, Debug)]
 enum PlotType {
     Scatter,
     Line,
@@ -132,11 +139,6 @@ enum PlotType {
     Text,
 }
 
-struct Plot<'a, X: PlotNum + 'a, Y: PlotNum + 'a> {
-    plot_type: PlotType,
-    plots: Box<dyn PlotTrait<X, Y> + 'a>,
-}
-
 ///
 /// Created once the min and max bounds of all the plots has been computed.
 /// Contains in it all the information typically needed to make a [`TickInfo`].
@@ -161,13 +163,13 @@ pub fn ticks_from_default<X: HasDefaultTicks>(bound: &Bound<X>) -> (TickInfo<X>,
 ///
 /// Created by [`Data::build`]
 ///
-pub struct DataResult<'a, X: PlotNum, Y: PlotNum> {
-    plots: Vec<Plot<'a, X, Y>>,
+pub struct DataResult<'a, X: PlotNum + 'a, Y: PlotNum + 'a> {
+    plots: Vec<Box<dyn PlotTrait<Item = (X, Y)> + 'a>>,
     canvas: render::Canvas,
     boundx: Bound<X>,
     boundy: Bound<Y>,
 }
-impl<'a, X: PlotNum, Y: PlotNum> DataResult<'a, X, Y> {
+impl<'a, X: PlotNum + 'a, Y: PlotNum + 'a> DataResult<'a, X, Y> {
     pub fn boundx(&self) -> &Bound<X> {
         &self.boundx
     }
@@ -291,14 +293,14 @@ use plotnum::PlotIter;
 ///
 /// Plot collector.
 ///
-pub struct Data<'a, X: PlotNum, Y: PlotNum> {
-    plots: Vec<Plot<'a, X, Y>>,
+pub struct Data<'a, X: PlotNum + 'a, Y: PlotNum + 'a> {
+    plots: Vec<Box<dyn PlotTrait<Item = (X, Y)> + 'a>>,
     xmarkers: Vec<X>,
     ymarkers: Vec<Y>,
     num_css_classes: Option<usize>,
     preserve_aspect: bool,
 }
-impl<'a, X: PlotNum, Y: PlotNum> Default for Data<'a, X, Y> {
+impl<'a, X: PlotNum + 'a, Y: PlotNum + 'a> Default for Data<'a, X, Y> {
     fn default() -> Self {
         Data {
             plots: vec![],
@@ -309,7 +311,7 @@ impl<'a, X: PlotNum, Y: PlotNum> Default for Data<'a, X, Y> {
         }
     }
 }
-impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
+impl<'a, X: PlotNum + 'a, Y: PlotNum + 'a> Data<'a, X, Y> {
     pub fn xmarker(&mut self, a: X) -> &mut Self {
         self.xmarkers.push(a);
         self
@@ -328,10 +330,11 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
     /// plotter.text("This is a note");
     /// ```
     pub fn text(&mut self, name: impl Display + 'a) -> &mut Self {
-        self.plots.push(Plot {
-            plot_type: PlotType::Text,
-            plots: Box::new(PlotStruct::new(std::iter::empty(), name)),
-        });
+        self.plots.push(Box::new(PlotStruct::new(
+            std::iter::empty(),
+            name,
+            PlotType::Text,
+        )));
         self
     }
 
@@ -346,12 +349,14 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
     pub fn line<I>(&mut self, name: impl Display + 'a, plots: I) -> &mut Self
     where
         I: PlotIter + 'a,
-        I::Item: Plottable<X, Y>,
+        I::Item1: Plottable<Item = (X, Y)>,
+        I::Item2: Plottable<Item = (X, Y)>,
     {
-        self.plots.push(Plot {
-            plot_type: PlotType::Line,
-            plots: Box::new(PlotStruct::new(plots.map_plot(|x| x.make_plot()), name)),
-        });
+        self.plots.push(Box::new(PlotStruct::new(
+            plots.map_plot(|x| x.make_plot(), |x| x.make_plot()),
+            name,
+            PlotType::Line,
+        )));
         self
     }
 
@@ -366,12 +371,14 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
     pub fn line_fill<I>(&mut self, name: impl Display + 'a, plots: I) -> &mut Self
     where
         I: PlotIter + 'a,
-        I::Item: Plottable<X, Y>,
+        I::Item1: Plottable<Item = (X, Y)>,
+        I::Item2: Plottable<Item = (X, Y)>,
     {
-        self.plots.push(Plot {
-            plot_type: PlotType::LineFill,
-            plots: Box::new(PlotStruct::new(plots.map_plot(|x| x.make_plot()), name)),
-        });
+        self.plots.push(Box::new(PlotStruct::new(
+            plots.map_plot(|x| x.make_plot(), |x| x.make_plot()),
+            name,
+            PlotType::LineFill,
+        )));
         self
     }
 
@@ -387,12 +394,14 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
     pub fn line_fill_raw<I>(&mut self, name: impl Display + 'a, plots: I) -> &mut Self
     where
         I: PlotIter + 'a,
-        I::Item: Plottable<X, Y>,
+        I::Item1: Plottable<Item = (X, Y)>,
+        I::Item2: Plottable<Item = (X, Y)>,
     {
-        self.plots.push(Plot {
-            plot_type: PlotType::LineFillRaw,
-            plots: Box::new(PlotStruct::new(plots.map_plot(|x| x.make_plot()), name)),
-        });
+        self.plots.push(Box::new(PlotStruct::new(
+            plots.map_plot(|x| x.make_plot(), |x| x.make_plot()),
+            name,
+            PlotType::LineFillRaw,
+        )));
         self
     }
 
@@ -409,12 +418,14 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
     pub fn scatter<I>(&mut self, name: impl Display + 'a, plots: I) -> &mut Self
     where
         I: PlotIter + 'a,
-        I::Item: Plottable<X, Y>,
+        I::Item1: Plottable<Item = (X, Y)>,
+        I::Item2: Plottable<Item = (X, Y)>,
     {
-        self.plots.push(Plot {
-            plot_type: PlotType::Scatter,
-            plots: Box::new(PlotStruct::new(plots.map_plot(|x| x.make_plot()), name)),
-        });
+        self.plots.push(Box::new(PlotStruct::new(
+            plots.map_plot(|x| x.make_plot(), |x| x.make_plot()),
+            name,
+            PlotType::Scatter,
+        )));
         self
     }
 
@@ -430,12 +441,14 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
     pub fn histogram<I>(&mut self, name: impl Display + 'a, plots: I) -> &mut Self
     where
         I: PlotIter + 'a,
-        I::Item: Plottable<X, Y>,
+        I::Item1: Plottable<Item = (X, Y)>,
+        I::Item2: Plottable<Item = (X, Y)>,
     {
-        self.plots.push(Plot {
-            plot_type: PlotType::Histo,
-            plots: Box::new(PlotStruct::new(plots.map_plot(|x| x.make_plot()), name)),
-        });
+        self.plots.push(Box::new(PlotStruct::new(
+            plots.map_plot(|x| x.make_plot(), |x| x.make_plot()),
+            name,
+            PlotType::Histo,
+        )));
         self
     }
 
@@ -492,7 +505,7 @@ impl<'a, X: PlotNum, Y: PlotNum> Data<'a, X, Y> {
         let mut val = self.move_into();
 
         let (boundx, boundy) = util::find_bounds(
-            val.plots.iter_mut().flat_map(|x| x.plots.iter_first()),
+            val.plots.iter_mut().flat_map(|x| x.iter_first()),
             val.xmarkers.clone(),
             val.ymarkers.clone(),
         );
@@ -541,7 +554,7 @@ pub struct Plotter<'a, X: PlotNum + 'a, Y: PlotNum + 'a> {
     ticky: TickInfo<Y>,
 }
 
-impl<'a, X: PlotNum, Y: PlotNum> Plotter<'a, X, Y> {
+impl<'a, X: PlotNum + 'a, Y: PlotNum + 'a> Plotter<'a, X, Y> {
     ///
     /// Use the plot iterators to write out the graph elements.
     /// Does not add a svg tag, or any styling elements.
